
#include <iostream>
#include <iomanip>
#include <math.h>
#include "Graphics.h"


Graphics g;
double* mat;
int W = 300; // default width 
using namespace  std;


// Method of selecting a wrapped neighbour
inline int per(int i, int limit, int add) { 
  return (i+limit+add) % (limit);
}


// Method that draws the ising lattice
void Draw(double* m, Graphics& gr) {
  
  for (int x=0;x<W;x++) 
    for (int y=0;y<W;y++) {
      int c = 128 + (int)m[x+y*W]*128;
      if (c>255) c = 255;
      if (c<0) c= 0;
      gr.pixel(x,y,c,c,c);
    } 
}

// method that performs a single random update of the ising lattice
void update(double* m, double temp) {

  // Select a random position
  int x = (rand()%(W));
  int y = (rand()%(W));

  // Calculates change in energy
  double deltaE = 2*m[x+y*W]*(m[per(x,W,-1)+y*W] + m[per(x,W,1)+y*W] + m[x+per(y,W,1)*W] + m[x+W*per(y,W,-1)]);
  double r = (rand()/(double)RAND_MAX);
  // performs the metropolis test
  if (r<exp(-deltaE/temp)) {
    m[x+y*W]*=-1; // Flips the spin
  }
}
int main(int a, char* args[])
{

  // Performs input check
  if (a==2) {
    W = atoi(args[1]);
  }
  else
    cout <<"Usage : ising <size of window>, like 'ising 100' or 'ising 500'. Default width is 300." << endl;


  // Opens a new window
  g.openWindow("2D ising model",W,W); 
  SDL_Event event;

  // Allocates the lattice
  mat = new double[W*W];

  // Initialize "empty" lattice

  for (int i=0;i<W*W;i++)
      mat[i] = 1.0;
      

  srand(time(NULL));
  double temp = 30; // starting temperature
  double max = temp; // for drawing purposes
  double t = 0.99; // temp_n+1 < temp_n

  //  Infinite loop
  while(1)
    {
 
      // perform 30K updates pr window update
      for (int i=0;i<30000;i++) {
        update(mat, temp     ); 
      }

      temp*=t; // modify the temperature

      if (temp<0.3) // if temperature is too low, increase the temperature again
	t = 1.02;
      if (temp>max) // if temperature is too high, decrease the temperature
	t = 0.99;

      // draw the lattice
      Draw(mat,g);

      // Draw temperature bar:
      for (int i=0;i<(((double)W/(double)max)*temp);i++)
	for (int j=0;j<25;j++) {
	  g.addpixel(j,W-i,200,200,200);
	}

      // present to screem
      g.flip();

      // quit if any key is pressed
      while ( SDL_PollEvent(&event) ) {
	switch (event.type) {
	case SDL_KEYDOWN:
	  exit(0);
	  
	}
      }
    }

  delete[] mat;
  return 0;
}
 
